# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
import re

from webob.multidict import MultiDict

from kinto.core.cornice.validators._colander import body_validator as colander_body_validator
from kinto.core.cornice.validators._colander import headers_validator as colander_headers_validator
from kinto.core.cornice.validators._colander import path_validator as colander_path_validator
from kinto.core.cornice.validators._colander import (
    querystring_validator as colander_querystring_validator,
)
from kinto.core.cornice.validators._colander import validator as colander_validator
from kinto.core.cornice.validators._marshmallow import body_validator as marshmallow_body_validator
from kinto.core.cornice.validators._marshmallow import (
    headers_validator as marshmallow_headers_validator,
)
from kinto.core.cornice.validators._marshmallow import path_validator as marshmallow_path_validator
from kinto.core.cornice.validators._marshmallow import (
    querystring_validator as marshmallow_querystring_validator,
)
from kinto.core.cornice.validators._marshmallow import validator as marshmallow_validator


__all__ = [
    "colander_validator",
    "colander_body_validator",
    "colander_headers_validator",
    "colander_path_validator",
    "colander_querystring_validator",
    "marshmallow_validator",
    "marshmallow_body_validator",
    "marshmallow_headers_validator",
    "marshmallow_path_validator",
    "marshmallow_querystring_validator",
    "extract_cstruct",
    "DEFAULT_VALIDATORS",
    "DEFAULT_FILTERS",
]


DEFAULT_VALIDATORS = []
DEFAULT_FILTERS = []


def extract_cstruct(request):
    """
    Extract attributes from the specified `request` such as body, url, path,
    method, querystring, headers, cookies, and returns them in a single dict
    object.

    :param request: Current request
    :type request: :class:`~pyramid:pyramid.request.Request`

    :returns: A mapping containing most request attributes.
    :rtype: dict
    """
    is_json = re.match("^application/(.*?)json$", str(request.content_type))

    if request.content_type in ("application/x-www-form-urlencoded", "multipart/form-data"):
        body = request.POST.mixed()
    elif request.content_type and not is_json:
        body = request.body
    else:
        if request.body:
            try:
                body = request.json_body
            except ValueError as e:
                request.errors.add("body", "", "Invalid JSON: %s" % e)
                return {}
            else:
                if not hasattr(body, "items") and not isinstance(body, list):
                    request.errors.add("body", "", "Should be a JSON object or an array")
                    return {}
        else:
            body = {}

    cstruct = {
        "method": request.method,
        "url": request.url,
        "path": request.matchdict,
        "body": body,
    }

    for sub, attr in (("querystring", "GET"), ("header", "headers"), ("cookies", "cookies")):
        data = getattr(request, attr)
        if isinstance(data, MultiDict):
            data = data.mixed()
        else:
            data = dict(data)
        cstruct[sub] = data

    return cstruct



"""
This module handles the conversion between colander object schemas and swagger
object schemas.
"""

from kinto.core.cornice_swagger.converters.parameters import ParameterConversionDispatcher
from kinto.core.cornice_swagger.converters.schema import TypeConversionDispatcher


def convert_schema(schema_node):
    dispatcher = TypeConversionDispatcher()
    converted = dispatcher(schema_node)

    return converted


def convert_parameter(location, schema_node, definition_handler=convert_schema):
    dispatcher = ParameterConversionDispatcher(definition_handler)
    converted = dispatcher(location, schema_node)

    return converted



import importlib
from string import Template

import cornice
import cornice_swagger
import pkg_resources
from pyramid.response import Response


# hardcode for now since that will work for vast majority of users
# maybe later add minified resources for behind firewall support?
ui_css_url = "https://cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.23.11/swagger-ui.css"
ui_js_bundle_url = "https://cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.23.11/swagger-ui-bundle.js"
ui_js_standalone_url = (
    "https://cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.23.11/swagger-ui-standalone-preset.js"
)


def swagger_ui_template_view(request):
    """
    Serves Swagger UI page, default Swagger UI config is used but you can
    override the callable that generates the `<script>` tag by setting
    `cornice_swagger.swagger_ui_script_generator` in pyramid config, it defaults
    to 'cornice_swagger.views:swagger_ui_script_template'

    :param request:
    :return:
    """
    script_generator = request.registry.settings.get(
        "cornice_swagger.swagger_ui_script_generator",
        "cornice_swagger.views:swagger_ui_script_template",
    )
    package, callable = script_generator.split(":")
    imported_package = importlib.import_module(package)
    script_callable = getattr(imported_package, callable)
    template = pkg_resources.resource_string("cornice_swagger", "templates/index.html").decode(
        "utf8"
    )

    html = Template(template).safe_substitute(
        ui_css_url=ui_css_url,
        ui_js_bundle_url=ui_js_bundle_url,
        ui_js_standalone_url=ui_js_standalone_url,
        swagger_ui_script=script_callable(request),
    )
    return Response(html)


def open_api_json_view(request):
    """
    :param request:
    :return:

    Generates JSON representation of Swagger spec
    """
    doc = cornice_swagger.CorniceSwagger(
        cornice.service.get_services(), pyramid_registry=request.registry
    )
    kwargs = request.registry.settings["cornice_swagger.spec_kwargs"]
    my_spec = doc.generate(**kwargs)
    return my_spec


def swagger_ui_script_template(request, **kwargs):
    """
    :param request:
    :return:

    Generates the <script> code that bootstraps Swagger UI, it will be injected
    into index template
    """
    swagger_spec_url = request.route_url("cornice_swagger.open_api_path")
    template = pkg_resources.resource_string(
        "cornice_swagger", "templates/index_script_template.html"
    ).decode("utf8")
    return Template(template).safe_substitute(
        swagger_spec_url=swagger_spec_url,
    )



import unittest
from unittest import mock

from kinto.core.testing import skip_if_no_prometheus

from .support import BaseWebTest


@skip_if_no_prometheus
class ViewsMetricsTest(BaseWebTest, unittest.TestCase):
    @classmethod
    def get_app_settings(cls, extras=None):
        settings = super().get_app_settings(extras)
        settings["includes"] += "\nkinto.plugins.prometheus"
        return settings

    def setUp(self):
        super().setUp()
        patch = mock.patch("kinto.plugins.prometheus.PrometheusService")
        self.mocked = patch.start()
        self.addCleanup(patch.stop)

    def test_metrics_have_matchdict_labels(self):
        self.app.put("/buckets/beers", headers=self.headers)
        self.app.put("/buckets/beers/groups/amateurs", headers=self.headers)
        self.app.put("/buckets/beers/collections/barley", headers=self.headers)
        self.app.put("/buckets/beers/collections/barley/records/abc", headers=self.headers)

        self.app.get("/buckets", headers=self.headers)
        self.app.get("/buckets/beers/collections", headers=self.headers)
        self.app.get("/buckets/beers/collections/barley/records", headers=self.headers)

        resp = self.app.get("/__metrics__")
        print(resp.text)
        self.assertIn(
            'request_size_sum{bucket_id="beers",collection_id="",endpoint="bucket-object",group_id="",record_id=""}',
            resp.text,
        )
        self.assertIn(
            'request_size_sum{bucket_id="beers",collection_id="",endpoint="group-object",group_id="amateurs",record_id=""}',
            resp.text,
        )
        self.assertIn(
            'request_summary_total{bucket_id="beers",collection_id="barley",endpoint="collection-object",group_id="",method="put",record_id="",status="201"}',
            resp.text,
        )
        self.assertIn(
            'request_duration_sum{bucket_id="beers",collection_id="barley",endpoint="record-object",group_id="",method="put",record_id="abc"}',
            resp.text,
        )

        self.assertIn(
            'request_summary_total{bucket_id="",collection_id="",endpoint="bucket-plural",group_id="",method="get",record_id="",status="200"}',
            resp.text,
        )
        self.assertIn(
            'request_summary_total{bucket_id="beers",collection_id="",endpoint="collection-plural",group_id="",method="get",record_id="",status="200"}',
            resp.text,
        )
        self.assertIn(
            'request_summary_total{bucket_id="beers",collection_id="barley",endpoint="record-plural",group_id="",method="get",record_id="",status="200"}',
            resp.text,
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
import warnings


__all__ = [
    "is_string",
    "to_list",
    "match_accept_header",
    "ContentTypePredicate",
    "current_service",
    "func_name",
]


def is_string(s):
    return isinstance(s, str)


def to_list(obj):
    """Convert an object to a list if it is not already one"""
    if not isinstance(obj, (list, tuple)):
        obj = [
            obj,
        ]
    return obj


def match_accept_header(func, context, request):
    """
    Return True if the request ``Accept`` header match
    the list returned by the callable specified in :param:func.

    Also attach the total list of possible accepted
    egress media types to the request.

    Utility function for performing content negotiation.

    :param func:
        The callable returning the list of acceptable
        internet media types for content negotiation.
        It obtains the request object as single argument.
    """
    acceptable = to_list(func(request))
    request.info["acceptable"] = acceptable
    return len(request.accept.acceptable_offers(acceptable)) > 0


def match_content_type_header(func, context, request):
    """
    Return True if the request ``Content-Type`` header match
    the list returned by the callable specified in :param:func.

    Also attach the total list of possible accepted
    ingress media types to the request.

    Utility function for performing request body
    media type checks.

    :param func:
        The callable returning the list of acceptable
        internet media types for request body
        media type checks.
        It obtains the request object as single argument.
    """
    supported_contenttypes = to_list(func(request))
    request.info["supported_contenttypes"] = supported_contenttypes
    return content_type_matches(request, supported_contenttypes)


def extract_json_data(request):
    warnings.warn("Use ``cornice.validators.extract_cstruct()`` instead", DeprecationWarning)
    from kinto.core.cornice.validators import extract_cstruct

    return extract_cstruct(request)["body"]


def extract_form_urlencoded_data(request):
    warnings.warn("Use ``cornice.validators.extract_cstruct()`` instead", DeprecationWarning)
    return request.POST


def content_type_matches(request, content_types):
    """
    Check whether ``request.content_type``
    matches given list of content types.
    """
    return request.content_type in content_types


class ContentTypePredicate(object):
    """
    Pyramid predicate for matching against ``Content-Type`` request header.
    Should live in ``pyramid.config.predicates``.

    .. seealso::
      http://docs.pylonsproject.org/projects/pyramid/en/latest/narr/hooks.html
      #view-and-route-predicates
    """

    def __init__(self, val, config):
        self.val = val

    def text(self):
        return "content_type = %s" % (self.val,)

    phash = text

    def __call__(self, context, request):
        return request.content_type == self.val


def func_name(f):
    """Return the name of a function or class method."""
    if isinstance(f, str):
        return f
    elif hasattr(f, "__qualname__"):  # pragma: no cover
        return f.__qualname__  # Python 3
    elif hasattr(f, "im_class"):  # pragma: no cover
        return "{0}.{1}".format(f.im_class.__name__, f.__name__)  # Python 2
    else:  # pragma: no cover
        return f.__name__  # Python 2


def current_service(request):
    """Return the Cornice service matching the specified request.

    :returns: the service or None if unmatched.
    :rtype: kinto.core.cornice.Service
    """
    if request.matched_route:
        services = request.registry.cornice_services
        pattern = request.matched_route.pattern
        name = request.matched_route.name
        # try pattern first, then route name else return None
        service = services.get(pattern, services.get("__cornice" + name))
        return service



import bcrypt
from pyramid import authentication as base_auth

from kinto.core import utils
from kinto.core.storage import exceptions as storage_exceptions

from .utils import (
    ACCOUNT_CACHE_KEY,
    ACCOUNT_POLICY_NAME,
)


def account_check(username, password, request):
    settings = request.registry.settings
    hmac_secret = settings["userid_hmac_secret"]
    cache_key = utils.hmac_digest(hmac_secret, ACCOUNT_CACHE_KEY.format(username))
    cache_ttl = int(settings.get("account_cache_ttl_seconds", 30))
    hashed_password = utils.hmac_digest(cache_key, password)

    # Check cache to see whether somebody has recently logged in with the same
    # username and password.
    cache = request.registry.cache
    cache_result = cache.get(cache_key)

    # Username and password have been verified previously. No need to compare hashes
    if cache_result == hashed_password:
        # Refresh the cache TTL.
        cache.expire(cache_key, cache_ttl)
        return True

    # Back to standard procedure
    parent_id = username
    try:
        existing = request.registry.storage.get(
            parent_id=parent_id, resource_name="account", object_id=username
        )
    except storage_exceptions.ObjectNotFoundError:
        return None

    hashed = existing["password"].encode(encoding="utf-8")
    pwd_str = password.encode(encoding="utf-8")
    # Check if password is valid (it is a very expensive computation)
    if bcrypt.checkpw(pwd_str, hashed):
        cache.set(cache_key, hashed_password, ttl=cache_ttl)
        return True


class AccountsAuthenticationPolicy(base_auth.BasicAuthAuthenticationPolicy):
    """Accounts authentication policy.

    It will check that the credentials exist in the account resource.
    """

    name = ACCOUNT_POLICY_NAME

    def __init__(self, *args, **kwargs):
        super().__init__(account_check, *args, **kwargs)

    def effective_principals(self, request):
        # Bypass default Pyramid construction of principals because
        # Pyramid multiauth already adds userid, Authenticated and Everyone
        # principals.
        return []



"""Converts from colander request chema to Swagger parameters."""

from kinto.core.cornice_swagger.converters.exceptions import NoSuchConverter


class ParameterConverter(object):
    _in = None

    def convert(self, schema_node, definition_handler):
        """
        Convert node schema into a parameter object.
        """

        converted = {"name": schema_node.name, "in": self._in, "required": schema_node.required}
        if schema_node.description:
            converted["description"] = schema_node.description

        if schema_node.default:
            converted["default"] = schema_node.default

        schema = definition_handler(schema_node)
        # Parameters shouldn't have a title
        schema.pop("title", None)
        converted.update(schema)

        if schema.get("type") == "array":
            converted["items"] = {"type": schema["items"]["type"]}

        return converted


class PathParameterConverter(ParameterConverter):
    _in = "path"

    def convert(self, schema_node, definition_handler):
        converted = super(PathParameterConverter, self).convert(schema_node, definition_handler)
        # Extract regex pattern from name
        template = converted["name"].split(":", 1)
        if len(template) == 2:
            converted["name"] = template[0]
            converted["pattern"] = template[1]

        return converted


class QueryParameterConverter(ParameterConverter):
    _in = "query"


class HeaderParameterConverter(ParameterConverter):
    _in = "header"


class BodyParameterConverter(ParameterConverter):
    _in = "body"

    def convert(self, schema_node, definition_handler):
        converted = {"name": schema_node.name, "in": self._in, "required": schema_node.required}
        if schema_node.description:
            converted["description"] = schema_node.description

        schema_node.title = schema_node.__class__.__name__
        schema = definition_handler(schema_node)
        converted["schema"] = schema

        return converted


class ParameterConversionDispatcher(object):
    converters = {
        "body": BodyParameterConverter,
        "path": PathParameterConverter,
        "querystring": QueryParameterConverter,
        "GET": QueryParameterConverter,
        "header": HeaderParameterConverter,
        "headers": HeaderParameterConverter,
    }

    def __init__(self, definition_handler):
        self.definition_handler = definition_handler

    def __call__(self, location, schema_node):
        converter_class = self.converters.get(location)
        if converter_class is None:
            raise NoSuchConverter()

        converter = converter_class()
        converted = converter.convert(schema_node, self.definition_handler)

        return converted



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
import json

from pyramid.i18n import TranslationString


class Errors(list):
    """Holds Request errors"""

    def __init__(self, status=400, localizer=None):
        self.status = status
        self.localizer = localizer
        super(Errors, self).__init__()

    def add(self, location, name=None, description=None, **kw):
        """Registers a new error."""
        allowed = ("body", "querystring", "url", "header", "path", "cookies", "method")
        if location != "" and location not in allowed:
            raise ValueError("%r not in %s" % (location, allowed))

        if isinstance(description, TranslationString) and self.localizer:
            description = self.localizer.translate(description)

        self.append(dict(location=location, name=name, description=description, **kw))

    @classmethod
    def from_json(cls, string):
        """Transforms a json string into an `Errors` instance"""
        obj = json.loads(string.decode())
        return Errors.from_list(obj.get("errors", []))

    @classmethod
    def from_list(cls, obj):
        """Transforms a python list into an `Errors` instance"""
        errors = Errors()
        for error in obj:
            errors.add(**error)
        return errors



import types

from zope.interface import Interface, implementer

from kinto.core import utils


class IMetricsService(Interface):
    """
    An interface that defines the metrics service contract.
    Any class implementing this must provide all its methods.
    """

    def timer(key):
        """
        Watch execution time.
        """

    def observe(self, key, value, labels=[]):
        """
        Observe a give `value` for the specified `key`.
        """

    def count(key, count=1, unique=None):
        """
        Count occurrences. If `unique` is set, overwrites the counter value
        on each call.

        `unique` should be of type ``list[tuple[str,str]]``.
        """


class NoOpTimer:
    def __call__(self, f):
        @utils.safe_wraps(f)
        def _wrapped(*args, **kwargs):
            return f(*args, **kwargs)

        return _wrapped

    def __enter__(self):
        pass

    def __exit__(self, *args, **kwargs):
        pass


@implementer(IMetricsService)
class NoOpMetricsService:
    def timer(self, key, value=None, labels=[]):
        return NoOpTimer()

    def observe(self, key, value, labels=[]):
        pass

    def count(self, key, count=1, unique=None):
        pass


def watch_execution_time(metrics_service, obj, prefix="", classname=None):
    """
    Decorate all methods of an object in order to watch their execution time.
    Metrics will be named `{prefix}.{classname}.{method}`.
    """
    classname = classname or utils.classname(obj)
    members = dir(obj)
    for name in members:
        method = getattr(obj, name)
        is_method = isinstance(method, types.MethodType)
        if not name.startswith("_") and is_method:
            statsd_key = f"{prefix}.{classname}"
            labels = [("method", name)]
            decorated_method = metrics_service.timer(statsd_key, labels=labels)(method)
            setattr(obj, name, decorated_method)


def listener_with_timer(config, key, func):
    """
    Add a timer with the specified `key` on the specified `func`.
    This is used to avoid evaluating `config.registry.metrics` during setup time
    to avoid having to deal with initialization order and configuration committing.
    """

    def wrapped(*args, **kwargs):
        metrics_service = config.registry.metrics
        if not metrics_service:
            # This only happens if `kinto.core.initialization.setup_metrics` is
            # not listed in the `initialization_sequence` setting.
            return func(*args, **kwargs)
        # If metrics are enabled, monitor execution time of listeners.
        with metrics_service.timer(key):
            return func(*args, **kwargs)

    return wrapped



import functools
import unittest
from unittest import mock

from pyramid.exceptions import ConfigurationError

from kinto.core.testing import get_user_headers, skip_if_no_prometheus
from kinto.plugins import prometheus

from .. import support


DATETIME_REGEX = r"^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{6}\+\d{2}:\d{2}$"


class PrometheusMissing(unittest.TestCase):
    def setUp(self):
        self.previous = prometheus.prometheus_module
        prometheus.prometheus_module = None

    def tearDown(self):
        prometheus.prometheus_module = self.previous

    def test_client_instantiation_raises_properly(self):
        with self.assertRaises(ConfigurationError):
            prometheus.includeme(mock.MagicMock())


class PrometheusWebTest(support.BaseWebTest, unittest.TestCase):
    @classmethod
    def get_app_settings(cls, extras=None):
        settings = super().get_app_settings(extras)
        settings["includes"] = "kinto.plugins.prometheus"
        settings["project_name"] = "kinto PROD"
        return settings


@skip_if_no_prometheus
class ViewsTest(PrometheusWebTest):
    def test_prometheus_capability_if_enabled(self):
        resp = self.app.get("/")
        capabilities = resp.json["capabilities"]
        self.assertIn("prometheus", capabilities)

    def test_endpoint_with_metrics(self):
        self.app.put("/buckets/test", headers=get_user_headers("aaa"))

        resp = self.app.get("/__metrics__")
        self.assertIn("Summary", resp.text)


def my_func(a, b):
    return a + b


@skip_if_no_prometheus
class ServiceTest(PrometheusWebTest):
    def test_timer_can_be_used_as_context_manager(self):
        with self.app.app.registry.metrics.timer("func.latency.context"):
            self.assertEqual(my_func(1, 1), 2)

        resp = self.app.get("/__metrics__")
        self.assertIn("TYPE kintoprod_func_latency_context histogram", resp.text)

    def test_timer_can_be_used_as_decorator(self):
        decorated = self.app.app.registry.metrics.timer("func.latency.decorator")(my_func)

        self.assertEqual(decorated(1, 1), 2)

        resp = self.app.get("/__metrics__")
        self.assertIn("TYPE kintoprod_func_latency_decorator histogram", resp.text)

    def test_timer_can_be_used_as_decorator_on_partial_function(self):
        partial = functools.partial(my_func, 3)
        decorated = self.app.app.registry.metrics.timer("func.latency.partial")(partial)

        self.assertEqual(decorated(3), 6)

        resp = self.app.get("/__metrics__")
        self.assertIn("TYPE kintoprod_func_latency_partial histogram", resp.text)

    def test_observe_a_single_value(self):
        self.app.app.registry.metrics.observe("price", 111)

        resp = self.app.get("/__metrics__")
        self.assertIn("kintoprod_price_sum 111", resp.text)

    def test_observe_a_single_value_with_labels(self):
        self.app.app.registry.metrics.observe("size", 3.14, labels=[("endpoint", "/buckets")])

        resp = self.app.get("/__metrics__")
        self.assertIn('kintoprod_size_sum{endpoint="/buckets"} 3.14', resp.text)

    def test_count_by_key(self):
        self.app.app.registry.metrics.count("key")

        resp = self.app.get("/__metrics__")
        self.assertIn("kintoprod_key_total 1.0", resp.text)

    def test_count_by_key_value(self):
        self.app.app.registry.metrics.count("bigstep", count=2)

        resp = self.app.get("/__metrics__")
        self.assertIn("kintoprod_bigstep_total 2.0", resp.text)

    def test_count_by_key_grouped(self):
        self.app.app.registry.metrics.count("http", unique=[("status", "500")])
        self.app.app.registry.metrics.count("http", unique=[("status", "200")])

        resp = self.app.get("/__metrics__")
        self.assertIn('kintoprod_http_total{status="500"} 1.0', resp.text)
        self.assertIn('kintoprod_http_total{status="200"} 1.0', resp.text)

    def test_metrics_cant_be_mixed(self):
        self.app.app.registry.metrics.count("counter")
        with self.assertRaises(RuntimeError):
            self.app.app.registry.metrics.timer("counter")

        self.app.app.registry.metrics.timer("timer")
        with self.assertRaises(RuntimeError):
            self.app.app.registry.metrics.count("timer")

        self.app.app.registry.metrics.count("observer")
        with self.assertRaises(RuntimeError):
            self.app.app.registry.metrics.observe("observer", 42)

    def test_metrics_names_and_labels_are_transformed(self):
        self.app.app.registry.metrics.count("http.home.status", unique=[("code.get", "200")])

        resp = self.app.get("/__metrics__")
        self.assertIn('kintoprod_http_home_status_total{code_get="200"} 1.0', resp.text)

    def test_count_with_legacy_string_generic_group(self):
        self.app.app.registry.metrics.count("champignons", unique="boletus")

        resp = self.app.get("/__metrics__")
        self.assertIn('kintoprod_champignons_total{group="boletus"} 1.0', resp.text)

    def test_count_with_legacy_string_basic_group(self):
        self.app.app.registry.metrics.count("mushrooms", unique="species.boletus")

        resp = self.app.get("/__metrics__")
        self.assertIn('kintoprod_mushrooms_total{species="boletus"} 1.0', resp.text)


@skip_if_no_prometheus
class PrometheusNoPrefixTest(PrometheusWebTest):
    @classmethod
    def get_app_settings(cls, extras=None):
        settings = super().get_app_settings(extras)
        settings["project_name"] = "Some Project"
        settings["prometheus_prefix"] = ""
        return settings

    def test_metrics_have_no_prefix(self):
        self.app.app.registry.metrics.observe("price", 111)

        resp = self.app.get("/__metrics__")
        self.assertIn("TYPE price summary", resp.text)
